@@ -1297,92 +1297,148 @@ class interp(machine_state):
 
     def run_spawn(self, jac_ast):
         """
-        spawn: KW_SPAWN expression spawn_object;
+        spawn: KW_SPAWN spawn_object;
 
         NOTE: spawn statements support locations that are either nodes or
         jac_sets
         """
         kid = self.set_cur_ast(jac_ast)
-        if(kid[1].name == 'expression'):
-            location = self.run_expression(kid[1]).value
-            if(isinstance(location, node)):
-                return self.run_spawn_object(kid[2], location)
-            elif(isinstance(location, jac_set)):
-                res = []
-                for i in location.obj_list():
-                    res.append(self.run_spawn_object(kid[2], i))
-                return jac_value(self, value=res)
-            else:
-                self.rt_error(
-                    f'Spawn can not occur on {type(location)}!', kid[1])
-        else:
-            return self.run_spawn_object(kid[1], None)
+        return self.run_spawn_object(kid[1])
 
-    def run_spawn_object(self, jac_ast, location):
+    def run_spawn_object(self, jac_ast):
         """
-        spawn_object: node_spawn | walker_spawn;
+        spawn_object: node_spawn | walker_spawn | graph_spawn;
         """
         kid = self.set_cur_ast(jac_ast)
-        return self.run_rule(kid[0], location)
+        return self.run_rule(kid[0])
 
-    def run_node_spawn(self, jac_ast, location):
+    def run_spawn_edge(self, jac_ast):
         """
-        node_spawn: edge_ref? node_ref spawn_ctx?;
+        spawn_edge: expression edge_ref;
+        """
+        kid = self.set_cur_ast(jac_ast)
+        loc = self.run_expression(kid[0]).value
+        if(isinstance(loc, jac_set)):
+            edge_set = [self.run_edge_ref(kid[1], is_spawn=True) for _ in loc]
+            loc = loc.obj_list()
+        else:
+            edge_set = self.run_edge_ref(kid[1], is_spawn=True)
+        return {
+            "location": loc,
+            "use_edge": edge_set,
+            "direction": kid[1].kid[0].name,
+        }
+
+    def run_node_spawn(self, jac_ast):
+        """
+        node_spawn: spawn_edge? node_ref spawn_ctx?;
         """
         kid = self.set_cur_ast(jac_ast)
         if(kid[0].name == 'node_ref'):
             ret_node = self.run_node_ref(kid[0], is_spawn=True)
+            if (kid[-1].name == 'spawn_ctx'):
+                self.run_spawn_ctx(kid[-1], ret_node)
+            return jac_value(self, value=ret_node)
         else:
-            use_edge = self.run_edge_ref(kid[0], is_spawn=True)
-            ret_node = self.run_node_ref(kid[1], is_spawn=True)
-            direction = kid[0].kid[0].name
-            if (direction == 'edge_from'):
-                location.attach_inbound(ret_node, [use_edge])
-            elif (direction == 'edge_to'):
-                location.attach_outbound(ret_node, [use_edge])
+            sp = self.run_spawn_edge(kid[0])
+            if(isinstance(sp['location'], node)):
+                ret_node = self.run_node_ref(kid[1], is_spawn=True)
+                if (sp['direction'] == 'edge_from'):
+                    sp['location'].attach_inbound(ret_node, [sp['use_edge']])
+                elif (sp['direction'] == 'edge_to'):
+                    sp['location'].attach_outbound(ret_node, [sp['use_edge']])
+                else:
+                    sp['location'].attach_bidirected(
+                        ret_node, [sp['use_edge']])
+                if (kid[-1].name == 'spawn_ctx'):
+                    self.run_spawn_ctx(kid[-1], ret_node)
+                return jac_value(self, value=ret_node)
+            elif(isinstance(sp['location'], jac_set)):
+                res = []
+                sp['location'] = sp['location'].obj_list()
+                for i in range(len(sp['location'])):
+                    ret_node = self.run_node_ref(kid[1], is_spawn=True)
+                    if (sp['direction'] == 'edge_from'):
+                        sp['location'][i].attach_inbound(
+                            ret_node, [sp['use_edge'][i]])
+                    elif (sp['direction'] == 'edge_to'):
+                        sp['location'][i].attach_outbound(
+                            ret_node, [sp['use_edge'][i]])
+                    else:
+                        sp['location'][i].attach_bidirected(
+                            ret_node, [sp['use_edge'][i]])
+                    if (kid[-1].name == 'spawn_ctx'):
+                        self.run_spawn_ctx(kid[-1], ret_node)
+                    res.append(ret_node)
+                return jac_value(self, value=res)
             else:
-                location.attach_bidirected(ret_node, [use_edge])
-        if (kid[-1].name == 'spawn_ctx'):
-            self.run_spawn_ctx(kid[-1], ret_node)
-        return jac_value(self, value=ret_node)
+                self.rt_error(
+                    f'Spawn can not occur on {type(sp["location"])}!', kid[1])
 
-    def run_walker_spawn(self, jac_ast, location):
+    def run_graph_spawn(self, jac_ast):
         """
-        walker_spawn: walker_ref spawn_ctx?;
-        """
-        kid = self.set_cur_ast(jac_ast)
-        walk = self.run_walker_ref(kid[0])
-        walk.prime(location)
-        if(len(kid) > 1):
-            self.run_spawn_ctx(kid[1], walk)
-        walk.run()
-        ret = jac_value(self, value=walk.anchor_value())
-        ret.unwrap()
-        self.report += walk.report
-        if(walk.report_status):
-            self.report_status = walk.report_status
-        self.runtime_errors += walk.runtime_errors
-        walk.destroy()
-        return ret
-
-    def run_graph_spawn(self, jac_ast, location):
-        """
-        graph_spawn: edge_ref? graph_ref;
+        graph_spawn: spawn_edge? graph_ref;
         """
         kid = self.set_cur_ast(jac_ast)
         if(kid[0].name == 'graph_ref'):
             result = self.run_graph_ref(kid[0])
+            return jac_value(self, value=result)
         else:
-            use_edge = self.run_edge_ref(kid[0], is_spawn=True)
-            result = self.run_graph_ref(kid[1])
-            direction = kid[0].kid[0].name
-            if (direction == 'edge_from'):
-                location.attach_inbound(result, [use_edge])
-            elif (direction == 'edge_to'):
-                location.attach_outbound(result, [use_edge])
+            sp = self.run_spawn_edge(kid[0])
+            if(isinstance(sp['location'], node)):
+                ret_node = self.run_graph_ref(kid[1])
+                if (sp['direction'] == 'edge_from'):
+                    sp['location'].attach_inbound(ret_node, [sp['use_edge']])
+                elif (sp['direction'] == 'edge_to'):
+                    sp['location'].attach_outbound(ret_node, [sp['use_edge']])
+                else:
+                    sp['location'].attach_bidirected(
+                        ret_node, [sp['use_edge']])
+                return jac_value(self, value=ret_node)
+            elif(isinstance(sp['location'], jac_set)):
+                res = []
+                sp['location'] = sp['location'].obj_list()
+                for i in range(len(sp['location'])):
+                    ret_node = self.run_graph_ref(kid[1])
+                    if (sp['direction'] == 'edge_from'):
+                        sp['location'][i].attach_inbound(
+                            ret_node, [sp['use_edge'][i]])
+                    elif (sp['direction'] == 'edge_to'):
+                        sp['location'][i].attach_outbound(
+                            ret_node, [sp['use_edge'][i]])
+                    else:
+                        sp['location'][i].attach_bidirected(
+                            ret_node, [sp['use_edge'][i]])
+                    res.append(ret_node)
+                return jac_value(self, value=res)
             else:
-                location.attach_bidirected(result, [use_edge])
-        return jac_value(self, value=result)
+                self.rt_error(
+                    f'Spawn can not occur on {type(sp["location"])}!', kid[1])
+
+    def run_walker_spawn(self, jac_ast):
+        """
+        walker_spawn: expression walker_ref spawn_ctx?;
+        """
+        kid = self.set_cur_ast(jac_ast)
+        location = self.run_expression(kid[0]).value
+        if(isinstance(location, node)):
+            location = jac_set(in_list=[location])
+        ret = []
+        for i in location.obj_list():
+            walk = self.run_walker_ref(kid[1])
+            walk.prime(i)
+            if(len(kid) > 2):
+                self.run_spawn_ctx(kid[2], walk)
+            walk.run()
+            tr = jac_value(self, value=walk.anchor_value())
+            tr.unwrap()
+            ret.append(tr.value)
+            self.report += walk.report
+            if(walk.report_status):
+                self.report_status = walk.report_status
+            self.runtime_errors += walk.runtime_errors
+            walk.destroy()
+        return jac_value(self, value=ret[0] if len(ret) == 1 else ret)
 
     def run_spawn_ctx(self, jac_ast, obj):
         """
